@@ -100,12 +100,12 @@ static inline int check_etag(connection_t* connection_prop,char *a) {
 This function does some changes on the URL.
 The url will never be longer than the original one.
 */
-static inline void modURL(char* url) {
+static inline void modURL(char* url, bool params) {
     replaceEscape(url);
 
     //Prevents the use of .. to access the whole filesystem
-    strReplace(url,"../",'\0');
-
+    if (!params)
+        strReplace(url,"../",'\0');
     //TODO AbsoluteURI: Check if the url uses absolute url, and in that case remove the 1st part
 }
 
@@ -127,8 +127,9 @@ static inline void set_connection_props(connection_t *connection_prop) {
         connection_prop->keep_alive=(connection && strncmp(a,"Keep",4)==0)?true:false;
     }
 
-    modURL(connection_prop->page);//Operations on the url string
     split_get_params(connection_prop);//Splits URI into page and parameters
+    modURL(connection_prop->page, false);//Operations on the url string
+    modURL(connection_prop->get_params, true);
     connection_prop->basedir=get_basedir(connection_prop->http_param);
 }
 
@@ -699,7 +700,18 @@ int write_dir(char* real_basedir,connection_t* connection_prop) {
 
     if ((pagelen=list_dir (connection_prop,html,MAXSCRIPTOUT,parent))<0) { //Creates the page
         free(html);//Frees the memory used for the page
-        return ERR_FILENOTFOUND;
+        switch (pagelen) {
+            case -1:
+                return ERR_FILENOTFOUND;
+            case -2:
+            default:
+#ifdef SERVERDBG
+                syslog(LOG_ERR, "Directory too large to be listed");
+#endif
+                return ERR_INSUFFICIENT_STORAGE;
+        }
+
+
     } else { //If there are no errors sends the page
 
         /*WARNING using the directory's mtime here allows better caching and
@@ -710,7 +722,14 @@ int write_dir(char* real_basedir,connection_t* connection_prop) {
         I tried on reiserfs and the directory's mtime changes too but i didn't
         find any doc about the other filesystems and OS.
         */
-        send_http_header(200,pagelen,NULL,true,connection_prop->strfile_stat.st_mtime,connection_prop);
+        send_http_header(
+            200,
+            pagelen,
+            "Content-Type: text/html\r\n",
+            true,
+            connection_prop->strfile_stat.st_mtime,
+            connection_prop
+        );
         write(sock,html,pagelen);
 
         //Write item in cache
