@@ -35,11 +35,106 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <unistd.h>
 #include <syslog.h>
 #include <signal.h>
+#include <string.h>
 
 #include "mystring.h"
 #include "utils.h"
 #include "embedded_auth.h"
 
+/**
+ * This function creates an URI encoded version of the string origin
+ **/
+static void uri_encode(char *dest, size_t size, const char *origin) {
+    if (size == 0 || dest == NULL || origin == NULL)
+        return;
+
+    dest[0] = '\0';
+
+    size_t len = strlen(origin);
+    size_t rlen = 0;
+
+    char *format;
+    for (size_t i = 0; i < len; i++) {
+        //Encode forbidden characters
+        if (
+            origin[i] == '!' ||
+            origin[i] == '*' ||
+            origin[i] == '\'' ||
+            origin[i] == '(' ||
+            origin[i] == ')' ||
+            origin[i] == ';' ||
+            origin[i] == ':' ||
+            origin[i] == '@' ||
+            origin[i] == '&' ||
+            origin[i] == '=' ||
+            origin[i] == '+' ||
+            origin[i] == '$' ||
+            origin[i] == ',' ||
+            origin[i] == '/' ||
+            origin[i] == '?' ||
+            origin[i] == '#' ||
+            origin[i] == '[' ||
+            origin[i] == ']' ||
+            origin[i] == '%' ||
+            origin[i] == '>' ||
+            origin[i] == '<' ||
+            origin[i] == '"' ||
+            origin[i] == ' ' ||
+            origin[i] == '\\'
+        ) {
+            format = "%%%02x";
+        } else {
+            format = "%c";
+        }
+        if (size - rlen <= 2)
+            return;
+        rlen += sprintf(dest + rlen, format, origin[i]);
+    }
+}
+
+/**
+ * This function encodes the characters in 'origin' to &code; form into 'encoded'
+ **/
+static void html_encode(char *dest, size_t size, const char *origin) {
+    if (size == 0 || dest == NULL || origin == NULL) return;
+
+    dest[0] = '\0';
+
+    size_t len = 0;
+    char *code;
+    size_t codesize;
+    char ni[2] = "X";
+
+    for (size_t i = 0; i < strlen(origin); i++) {
+        ni[0] = origin[i];
+        switch (origin[i]) {
+        case '&':
+            code = "&amp;";
+            break;
+        case '<':
+            code = "&lt;";
+            break;
+        case '>':
+            code = "&gt;";
+            break;
+        case ' ':
+            code = "&nbsp;";
+            break;
+        case '"':
+            code = "&quot;";
+            break;
+        default:
+            code = ni;
+            break;
+        }
+        codesize = strlen(code);
+        if (len + codesize >= size)
+            return;
+        strcpy(dest + len, code);
+        len += codesize;
+    }
+}
+
 /**
 This function reads the directory dir, putting inside the html string an html
 page with links to all the files within the directory.
@@ -47,6 +142,12 @@ page with links to all the files within the directory.
 Buffer for html must be allocated by the calling function.
 bufsize is the size of the buffer allocated for html
 parent is true when the dir has a parent dir
+
+Returns the size of the HTML.
+
+-1: unable to open the file
+-2: out of memory
+
 */
 int list_dir(connection_t *connection_prop, char *html, unsigned int bufsize, bool parent) {
     int pagesize=0; //Written bytes on the page
@@ -55,15 +156,19 @@ int list_dir(connection_t *connection_prop, char *html, unsigned int bufsize, bo
     char *color; //Depending on row count chooses a background color
     char *measure; //contains measure unit for file's size (B, KiB, MiB)
     int counter = 0;
+    int errcode = 0;
 
     char path[INBUFFER]; //Buffer to contain element's absolute path
 
-    struct dirent **namelist;
+    struct dirent **namelist = NULL;
     counter = scandir(connection_prop->strfile, &namelist, 0, alphasort);
+    char *name_html = malloc(ESCAPED_FNAME_LEN);
+    char *escaped_dname = malloc(ESCAPED_FNAME_LEN);
 
 
     if (counter <0) { //Open not succesfull
-        return -1;
+        errcode = -1;
+        goto escape;
     }
 
     //Specific header table)
@@ -85,7 +190,7 @@ int list_dir(connection_t *connection_prop, char *html, unsigned int bufsize, bo
 
     for (i=0; i<counter; i++) {
         //Skipping hidden files
-        if (namelist[i]->d_name[0] == '.') {
+        if (namelist[i]->d_name[0] == '.' || errcode) {
             free(namelist[i]);
             continue;
         }
@@ -93,7 +198,6 @@ int list_dir(connection_t *connection_prop, char *html, unsigned int bufsize, bo
         snprintf(path, INBUFFER,"%s/%s", connection_prop->strfile, namelist[i]->d_name);
 
         //Stat on the entry
-
         stat(path, &f_prop);
         int f_mode = f_prop.st_mode; //Get's file's mode
 
@@ -101,6 +205,9 @@ int list_dir(connection_t *connection_prop, char *html, unsigned int bufsize, bo
         localtime_r(&f_prop.st_mtime,&ts);
         strftime(last_modified,URI_LEN, "%a, %d %b %Y %H:%M:%S", &ts);
 
+        html_encode(name_html, ESCAPED_FNAME_LEN, namelist[i]->d_name);
+        uri_encode(escaped_dname, ESCAPED_FNAME_LEN, namelist[i]->d_name);
+
         if (S_ISREG(f_mode)) { //Regular file
 
             //Table row for the file
@@ -122,10 +229,9 @@ int list_dir(connection_t *connection_prop, char *html, unsigned int bufsize, bo
                 color = "white";
             else
                 color = "#EAEAEA";
-
             printf_s=snprintf(html+pagesize,maxsize,
                               "<tr style=\"background-color: %s;\"><td>f</td><td><a href=\"%s\">%s</a></td><td>%lld%s</td><td>%s</td></tr>\n",
-                              color, namelist[i]->d_name, namelist[i]->d_name, (long long int)size, measure,last_modified);
+                              color, escaped_dname, name_html, (long long int)size, measure,last_modified);
             maxsize-=printf_s;
             pagesize+=printf_s;
 
@@ -133,20 +239,27 @@ int list_dir(connection_t *connection_prop, char *html, unsigned int bufsize, bo
             //Table row for the dir
             printf_s=snprintf(html+pagesize,maxsize,
                               "<tr style=\"background-color: #DFDFDF;\"><td>d</td><td><a href=\"%s/\">%s/</a></td><td>-</td><td>%s</td></tr>\n",
-                              namelist[i]->d_name, namelist[i]->d_name,last_modified);
+                              escaped_dname, name_html,last_modified);
             maxsize-=printf_s;
             pagesize+=printf_s;
         }
 
         free(namelist[i]);
+        if (maxsize <= 0) {
+            errcode = -2; // Out of memory
+        }
     }
 
+escape:
+    free(name_html);
+    free(escaped_dname);
     free(namelist);
-
-    printf_s=snprintf(html+pagesize,maxsize,"</table>%s",HTMLFOOT);
-    pagesize+=printf_s;
-
-    return pagesize;
+    if (errcode == 0) {
+        printf_s=snprintf(html+pagesize,maxsize,"</table>%s",HTMLFOOT);
+        pagesize+=printf_s;
+        return pagesize;
+    } else
+        return errcode;
 }
 
 /**
@@ -245,8 +358,9 @@ void help() {
            "  -p, --port    followed by port number to listen\n"
            "  -T  --inetd   must be specified when using weborf with inetd or xinetd\n"
            "  -t  --tar     will send the directories as .tar.gz files\n"
-           "  -u,           followed by a valid uid\n"
+           "  -u  --uid     followed by a valid uid\n"
            "                If started by root weborf will use this user to read files and execute scripts\n"
+           "  -g  --gid     followed by a valid gid\n"
            "  -V, --virtual list of virtualhosts in the form host=basedir, comma-separated\n"
            "  -v, --version print program version\n"
            "  -x, --noexec  tells weborf to send each file instead of executing scripts\n\n"
